/**
 * plugin.min.js
 *
 * Released under Attribution-ShareAlike 4.0 International License.
 * Author: Ignacio Gros (http://gros.es/) for http://exelearning.net/
 *
 * License: http://creativecommons.org/licenses/by-sa/4.0/
 */

/**
 * TinyMCE plugin to create definition lists
 * Don't forget to add some styles to theme/site so the code has the same appearance everywhere (see eXe's base.css file)
 * You may also want to add some JavaScript effects (see eXe's common.js file)
 */
tinymce.PluginManager.add('definitionlist', function(editor, url) {
	
	function addListOrTerm() {
			var se = editor.selection;

			var term = "<dt>"+_('Term')+"</dt>\n<dd><p>"+_('Term definition')+"</p></dd>\n";
			
			// No selection and not in DL
			var p = editor.dom.getParent(se.getNode(), 'DL');
			if (se.isCollapsed() && !p) {
				editor.windowManager.confirm(
					_('A new Definition List will be created.\nTo add a new term, just place the cursor at the end of a definition and click again the Definition List button.\n'), 
					function(s){
						if (s) {
							var html = '<dl class="exe-dl">\n';
								html += term;
								html += "</dl><br />";
							tinyMCE.execCommand('mceInsertContent', false, html);
						}
					}
				);
			} else {
				
				var n = editor.dom.getNext(se.getNode(), 'DT');
				if (n) $(n).before($(term));
				else $(p).append($(term));
				
				p = editor.dom.getParent(se.getNode(), 'DL');
				var DTs = p.getElementsByTagName("DT");
				for (i=0;i<DTs.length;i++) {
					if(DTs[i].innerHTML==_('Term')) {
						editor.selection.setCursorLocation(DTs[i],1);
					}
				}
			}  		
	}
	
	editor.addButton('definitionlist', {
		tooltip: _('Definition List'),
		image: url + '/img/definitionlist.png',
		onclick: function() {
			addListOrTerm();
		}
	});
	
	editor.addMenuItem('definitionlist', {
		text: _('Definition List'),
		image: url + '/img/definitionlist.png',
		context: 'insert',
		onclick: function() {
			addListOrTerm();
		}
	});
	
	editor.on('init', function(e) {
		editor.dom.loadCSS(url + "/css/content.css");
	});	
	
});